@@ -7,4 +7,10 @@ class MyHandler(WPHandler):                         # 1
     def hello(self, request, relpath):              # 2
         return "Hello, World!\n"                    # 3
 
-WPApp(MyHandler).run_server(8888)                   # 4
+import sys
+
+port = 8080
+if sys.argv[1:]:
+    port = int(sys.argv[1])
+print("Starting on port", port)
+WPApp(MyHandler).run_server(port)                   # 4
